/*
 Author: Tonetta Ivan (Ruben) - 2016.03 - Italy - Cabbage 1.02
- many thanks to God [Yahwéh, YHWH, Geova]: He made life possible, and all you can do with it;
- many thanks to my wife and my children for their patience;
- many thanks to Rory Walsh for Cabbage and his great support (http://forum.cabbageaudio.com/);
- many thanks to Iain McCurdy, for his great support and all code shared (http://iainmccurdy.org/);
- many thanks to all users presents in the Cabbage forum: t_grey, Guillermo Senna in particular for your support and all code shared (http://forum.cabbageaudio.com/)
- many thanks to Riccardo Bianchini, Alessandro Cipriani, Giorgio Zucco, Jim Aikin for their wonderful books and "Vallste" for his (old) support (http://triceratupuz.altervista.org/);
- many thanks to Barry Vercoe, Richard Boulanger, Max Mathews and all Csound developers in the world (http://www.csounds.com/);

I'm not a writer of professional code in csound; so what I present is a very simple Vox Continental emulator (...) (I like to make classical instruments you can play)
*/

<Cabbage>
form caption("CabVox") size(610, 225), colour("black"), pluginID("CbO2"), svgpath(".")

groupbox bounds(004,004,170,180), text(""), colour(128,000,000,020), linethickness(0), plant("Global"){ 
vslider  bounds(010,010,050,150), text("Master"),   channel("master"),  range( 0,      1,   .2,    .5,  .001), trackercolour(128,000,000,255)
vslider  bounds(060,010,050,150), text("Clic"),     channel("cliclvl"), range( 0,     .3,    0,    0.5, .001), trackercolour(128,000,000,255)
vslider  bounds(110,010,050,150), text("Vibrato"),  channel("vibdpth"), range( 0,    .03,    0,    0.5, .001), trackercolour(128,000,000,255)
}

groupbox bounds(180,004,150,180), text(""), colour(128,000,000,020), linethickness(0), plant("Partials"){ 
vslider  bounds(-50,-140,050,150), channel("a1"), trackercolour(128,000,000,255), range(0, 1, .5), trackerthickness(.25), gradient(1), rotate(3.14,050,150)
vslider  bounds(-15,-140,050,150), channel("a2"), trackercolour(128,000,000,255), range(0, 1, 0),  trackerthickness(.25), gradient(1), rotate(3.14,050,150)
vslider  bounds(020,-140,050,150), channel("a3"), trackercolour(128,000,000,255), range(0, 1, 0),  trackerthickness(.25), gradient(1), rotate(3.14,050,150)
vslider  bounds(055,-140,050,150), channel("a4"), trackercolour(128,000,000,255), range(0, 1, .2), trackerthickness(.25), gradient(1), rotate(3.14,050,150)
}
groupbox bounds(336,004,270,180), text(""), colour(128,000,000,020), linethickness(0), plant("EFX"){ 
vslider  bounds(010,010,050,150), text("Reverb"), channel("reverb"), range(0,  .1, .1),          trackercolour(128,000,000,255)
vslider  bounds(060,010,050,150), text("Room"),   channel("room"),   range(0,  1,  .1),          trackercolour(128,000,000,255)
vslider  bounds(110,010,050,150), text("Damph"),  channel("dmph"),   range(0,  1,  .1),          trackercolour(128,000,000,255)
vslider  bounds(160,010,050,150), text("Speed"),  channel("speed"),  range( 0, 10, 1, .5, .001), trackercolour(128,000,000,255)
vslider  bounds(210,010,050,150), text("Depth"),  channel("depth"),  range( 0, 10, 1, .5, .001), trackercolour(128,000,000,255)
}
groupbox   bounds(004,190,495,027), linethickness(0), colour(128,000,000,020), plant("Background"){ 
combobox   bounds(170,004,275,020), channel("combobox"), populate("*.snaps")
filebutton bounds(450,004,040,020), channel("save"), text("Save"), populate("*.snaps"), fontcolour(128,000,000,255)
label      bounds(004,006,180,015), text("Tonetta Ivan - 2016.03"), align("left"), fontcolour(128,000,000,255)
}
</Cabbage>  
<CsoundSynthesizer>
<CsOptions>
-dm0 -n -+rtmidi=NULL -M0 ;--midi-velocity-amp=5 ;--midi-key-cps=4
</CsOptions>
<CsInstruments>
sr 		    = 	44100
ksmps 		= 	16
nchnls 		= 	2
0dbfs		=	1

gisine ftgen 1, 0, 65536, 7, -1, 32768, 1, 32768, -1 ; high resolution Triangle wave for oscili
gidist ftgen 2, 0, 4096, 10, 1

; declaration/initialisation step for a global efx variable 
gaefxL init 0
gaefxR init 0

instr 100 ; Vox Instrument
	; receiving GUI values
	kmaster  chnget "master"
	ka1      chnget "a1"
	ka2      chnget "a2"
	ka3      chnget "a3"
	ka4      chnget "a4"
	kcliclvl chnget "cliclvl"
	ibutton1 chnget "button1"
	;kvibrato chnget "vibrato"
	kvibdpth chnget "vibdpth"
	; generating vibrato algorithm
	kvibrato oscil kvibdpth, 7, 2
	; the MIDI notes numbers and the Pitch-Bender values (+-12 semitones range) are releveated and converted in cps for oscili
	kmidi cpsmidib 12
	; Click's Envelope
	aclick madsr .0000000000001, .02, 0, .1, 0, 1
	; reading velocity value for "animate" the click signal
	ivelo veloc 0, 10
	; rescaling click noise to velocity amount
	aclick = aclick * ivelo
	; Click at note On
	aclick noise aclick, 0.999
	; GUI Control amplitude for Click signal
	anoion = aclick * kcliclvl
	; calculate the wright different cps for each oscillator
	knote1 =  kmidi       * (1 + kvibrato)
	knote2 =  (kmidi * 2) * (1 + kvibrato)
	knote3 =  (kmidi * 4) * (1 + kvibrato)
	knote4 =  (kmidi * 6) * (1 + kvibrato)
	knote5 =  (kmidi * 8) * (1 + kvibrato)
	; Amplitude envelope generation (NOTE: line at a rate because at k rate noise are present in the final audio mix)
	aenv1 madsr .0000000000001, .1, 1, .1, 0, 1
	; Audio generation
	a1 oscili aenv1 * ka1, knote1, 1
	a2 oscili aenv1 * ka2, knote2, 1
	a3 oscili aenv1 * ka3, knote3, 1
	a4 oscili aenv1 * ka4, knote4, 1
	a5 oscili aenv1 * ka4, knote5, 1
	; mixing all audio signals from oscili
	asum sum a1, a2, a3, a4, a5
	; rescale the amplitude of summing operation
	asum *= .25	
	; smoothing master knob values
	amaster	interp	kmaster
	; mixing all signals
	asignal = (asum + anoion) * amaster
	; Send STEREO audio signals ONLY to efx instruments below AFTER the master volume control knob!
	gaefxL += asignal
	gaefxR += asignal
endin

; Chorus Instrument STEREO
instr 998
	; Receiving GUI values
	kspeed chnget "speed"
	kdepth chnget "depth"
	; generating a line for smoothing values
	kporttime	linseg	0,0.001,0.05
	; Smooth k values
	kspeed portk kspeed,    kporttime
	kdepth portk kdepth,    kporttime
	; Generating 4 delay for L channel
	arand1 randomi .01, kdepth, kspeed
	a1 vdelay  gaefxL, arand1, 1000
	arand2 randomi .01, kdepth * .1, kspeed * .1
	a2 vdelay  gaefxL, arand2, 1000
	arand3 randomi .01, kdepth * .2, kspeed * .2
	a3 vdelay  gaefxL, arand3, 1000
	arand4 randomi .01, kdepth * .3, kspeed * .3
	a4 vdelay  gaefxL, arand4, 1000
	; Generating 4 delay for R channel
	arand5 randomi .01, kdepth * .4, kspeed * .4
	a5 vdelay  gaefxR, arand5, 1000
	arand6 randomi .01, kdepth * .5, kspeed * .5
	a6 vdelay  gaefxR, arand6, 1000
	arand7 randomi .01, kdepth * .6, kspeed * .6
	a7 vdelay  gaefxR, arand7, 1000
	arand8 randomi .01, kdepth * .7, kspeed * .7
	a8 vdelay  gaefxR, arand8, 1000
	; Summing all the fignals for each channel with amplitude adjustment
	asumL = (a1 + a2 + a3 + a4) * .1
	asumR = (a5 + a6 + a7 + a8) * .1
	; Audio Out
	outs asumL, asumR
	; send the same STEREO distortion signal to the last reverb Instrument below
	gaefxL += asumL
	gaefxR += asumR
endin

; Reverb Instrument STEREO
instr 999
	; receiving GUI values
	kroom chnget "room"
	kdmph chnget "dmph"
	kmix  chnget "reverb"
	; generating a line for smoothing values
	kporttime	linseg	0,0.001,0.05
	; Smooth k values
	kroom portk kroom,    kporttime
	kdmph portk kdmph,    kporttime
	; Reverb
	arL, arR  freeverb gaefxL, gaefxR, kroom, kdmph, sr, 0
	; smoothing Mix knob values
	amix	interp	kmix
	; Audio out with GUI gain control
	outs arL * amix, arR * amix
	; reset globals variables
	gaefxL = 0 
	gaefxR = 0 
endin
		
</CsInstruments>
<CsScore>
i 999 0 [3600*24] ; All the FX Instruments are active for 1 day
i 998 0 [3600*24]
</CsScore>
</CsoundSynthesizer>